;assembled with tniasm

cpu	z80

include "ym2610.inc"

;RESET
forg $00
	jp	Reset

;write to port A
org $08
forg $08
	push	af
	ld	a,d
	out	($04),a
	rst	$28
	ld	a,e
	out	($05),a
	rst	$28
	pop	af
	ret

;write to port B
org $18
forg $18
	push	af
	ld	a,d
	out	($06),a
	rst	$28
	ld	a,e
	out	($07),a
	rst	$28
	pop	af
	ret

;busy wait (MAME doesn't care)
org $28
forg $28
Busy:
	in	a,($04)		;wait until busy flag clear (bit7)
	add	a
	jr	c,Busy
	ret
	
;-----

;NMI:
org $66
forg $66
	in	a,($00)		;read 68k command
	cp	$01		;BIOS $01: Z80 error if this isn't replied to
	jr	z,.reply
	cp	$03		;BIOS $03: initialise, but this isn't needed
	jr	z,.return

	sub	$10		;anything else is a play command, values $10-$13 are valid
	call	PlaySample	;play the sample requested by 68k
	xor	a
.return:
	retn
.reply:
	out	($0C),a		;echo $01 byte
	retn

;-----

org $100
forg $100

Reset:
	ld	sp,$0000	;stack starts from end of RAM
	di			;no IRQ from YM2610
	out	($08),a		;enable NMI
.loop:
	jr	.loop		;Z80 just waits for 68k to send it stuff

;-----

;IN:
;A: sample # to play

MVOL:	equ $38		;master volume: $38
CVOL:	equ $1C+$C0	;channel volume: $1C, play on left and right channels

PlaySample:
	ld	b,a			;save sample # in reg b, which also serves as channel #
	add	a			;x4, 4 bytes per sample in SampleTable
	add	a
	ld	ix,SampleTable		;address of table to read pointers from
	ld	d,$00
	ld	e,a
	add	ix,de			;HL: address of sample pointers
	ld	de,PA_MVOL<<8 +MVOL	;set master volume
	rst	$18
	ld	a,PA_CVOL		;set left/right, channel volume
	add	b
	ld	d,a
	ld	e,CVOL
	rst	$18
	ld	a,PA_STARTL		;start address low byte
	add	b			;add channel #
	ld	d,a	
	ld	e,(ix+$00)		;read byte from sample address table
	rst	$18
	ld	a,PA_STARTH		;start address high byte
	add	b
	ld	d,a
	ld	e,(ix+$01)
	rst	$18
	ld	a,PA_ENDL		;end address low byte
	add	b
	ld	d,a
	ld	e,(ix+$02)
	rst	$18
	ld	a,PA_ENDH		;end address high byte
	add	b
	ld	d,a
	ld	e,(ix+$03)
	rst	$18
	ld	d,PA_CTRL		;play channel
	xor	a			;clear A
	scf
	inc	b			;shift count = channel + 1
.loop:
	adc	a			;"1 << channelnumber"
	djnz	.loop

	ld	e,a			;set key-on bit for this channel
	rst	$18	
	ret

;start and end addresses for each sample
SampleTable:
	dw $0000,$0026	;snare
	dw $0027,$0036	;bass
	dw $0037,$0061	;open hi-hat
	dw $0062,$006F	;closed hi-hat

	ds   $20000-$	;pad to 128K