
	cpu 68000
	supmode on

	include "reg.inc"

;RAM
	org $100000
vblankflag:	ds.w 1	;signal for main/vblank processing

;ROM header
	org $64
	dc.l	VBLANK	;IRQ handler

	org $0100
	dc.b "NEO-GEO",$00

	org $0108
	dc.w $017	;Sengoku

	org $0114
	dc.w $0100	;logo flag, don't show it just go straight to the entry point

	org $0122
	jmp Start	;entry

	org $182
	dc.l Code	;code pointer
Code:
    dc.l $76004A6D,$0A146600,$003C206D,$0A043E2D
    dc.l $0A0813C0,$00300001,$32100C01,$00FF671A
    dc.l $30280002,$B02D0ACE,$66103028,$0004B02D
    dc.l $0ACF6606,$B22D0AD0,$67085088,$51CFFFD4
    dc.l $36074E75,$206D0A04,$3E2D0A08,$3210E049
    dc.l $0C0100FF,$671A3010,$B02D0ACE,$66123028
    dc.l $0002E048,$B02D0ACF,$6606B22D,$0AD06708
    dc.l $588851CF,$FFD83607
    dc.w $4e75

;-----

;Z80 sample codes written to $320000
SNARE	equ $10
BASS	equ $11
OPNHAT	equ $12
CLSHAT	equ $13

Start:
	jsr	Display			;write text onscreen
	and	#$F0FF,SR		;allow interrupts now
.loop
	move.b	BIOS_P1CHANGE,d0	;controller ABCD decides what sound to play
.testA:
	btst	#CNT_A,d0		;button A is pressed?
	beq	.testB
	move.b	#SNARE,REG_SOUND	;then play snare
	jsr	Z80Wait
.testB:
	btst	#CNT_B,d0		;button B is pressed?
	beq	.testC
	move.b	#BASS,REG_SOUND		;then play bass drum
	jsr	Z80Wait
.testC:
	btst	#CNT_C,d0		;button C is pressed?
	beq	.testD
	move.b	#OPNHAT,REG_SOUND	;then play open hi-hats
	jsr	Z80Wait
.testD:
	btst	#CNT_D,d0		;button D is pressed?
	beq	.finished
	move.b	#CLSHAT,REG_SOUND	;then play closed hi-hats
	jsr	Z80Wait
.finished
	st	vblankflag		;signal that main loop is processed
.spin
	tst.b	vblankflag		;wait until vblank happens
	bne	.spin
	bra	.loop

;---

;give time for Z80 to process command

Z80Wait:
	move.w	d0,-(a7)
	move.w	#1000,d0		;arbitrary delay
zloop:
	dbra	d0,zloop
	move.w	(a7)+,d0
	rts

;---

;print a message onscreen

XPOS	SET 8
YPOS	SET 14

Display:
	clr.w	$400000			;black blanking color
	move.l	#$7FFF0000,$400002	;just show some text onscreen
	lea	VRAM_RW,a1
	move.w	#$0020,d0
	move.w	#$0500-1,d1
	move.w	#1,2(a1)
	move.w	#$7000,-2(a1)
.cloop:
	move.w	d0,(a1)			;clear screen first
	dbra	d1,.cloop

	move.w	#$20,2(a1)
	move.w	#$7000+XPOS*32+YPOS,-2(a1)
	lea	.text,a0
	clr.w	d0
	move.b	(a0)+,d0
.dloop:
	move.w	d0,(a1)			;then write string
	move.b	(a0)+,d0
	bne	.dloop

	rts
.text
	dc.b "Push buttons to play...",0

;---

;generic vblank handler, does nothing special

VBLANK:
	move.b	D0,$300001	;watchdog
	tst.b	$10FD80		;BIOS wants vblank?
	bmi	.regular
	jmp	$C00438
.regular
	move.w	#4,REG_IRQACK
	tst.b	vblankflag	;don't do anything if main routine hasn't finished yet
	beq	.return
.process
	movem.l	D0-D7/A0-A6,-(A7)		
	jsr	BIOSF_VBL	;read controller etc.
	movem.l	(A7)+,D0-D7/A0-A6
	clr.b	vblankflag	;signal that vblank is processed
.return
	rte
